require 'wouter'
require 'rack'
require 'tilt'

class Wouter
  class Views
    class FileNotFound < StandardError; end

    class <<self
      def file_finder(dir, template, engine, layout, bind)
        template_full_path = full_path(template, dir, engine)
        template_data = if Tilt.template_for(template_full_path)
                          Tilt.new(template_full_path).render(bind)
                        else
                          File.read(template_full_path)
                        end

        if layout
          layout_full_path = full_path(layout, dir, engine)
          if Tilt.template_for(layout_full_path)
            Tilt.new(layout_full_path).render(bind) { template_data }
          else
            File.read(layout_full_path) + template_data
          end
        else
          template_data
        end
      end

      def full_path(file, dir, engine)
        file_name = file.to_s + '.' + engine.to_s
        full_path = dir + '/' + file_name

        return full_path if File.exists?(full_path)

        full_path = dir + '/' + file.to_s

        return full_path if File.exists?(full_path)

        raise FileNotFound, "cannot find file: '#{file}' in '#{dir}'"
      end
    end

  end
end

class Wouter
  class <<self
    def render_template(template, bind = self, engine = nil, dir = nil, layout = nil)
      dir = dir || config[:view_dir] || File.expand_path('./views')
      engine = engine || config[:view_engine] || :html
      layout = layout.nil? ? config[:view_layout] : layout
      Wouter::Views.file_finder(dir, template, engine, layout, bind)
    end

    def render_view(template, engine: nil, dir: nil, layout: nil)
      ViewWrap.new(template, engine, dir, layout)
    end

    def view_dir(dir = nil)
      config[:view_dir] = dir if dir
      config[:view_dir]
    end

    def view_engine(engine = nil)
      config[:view_engine] = engine if engine
      config[:view_engine]
    end

    def view_layout(layout = nil)
      config[:view_layout] = layout if layout
      config[:view_layout]
    end
  end

  class ViewEndpoint < Wouter::Endpoint
    def initialize(req, res)
      @view = req.env['wouter.view']
      super(req, res)
    end

    def respond
      render_view(@view[:template], engine: @view[:engine], dir: @view[:dir], layout: @view[:layout])
    end
  end

  class ViewWrap
    def initialize(template, engine = nil, dir = nil, layout = nil)
      @template = template
      @engine = engine
      @dir = dir
      @layout = layout
    end

    def call(env)
      env['wouter.view'] = {
        template: @template,
        engine: @engine,
        dir: @dir,
        layout: @layout
      }
      ViewEndpoint.call(env)
    end
  end

  class Endpoint
    def render_view(template, engine: nil, dir: nil, layout: nil)
      engine ||= config[:view_engine]
      dir ||= config[:view_dir]
      layout = layout.nil? ? config[:view_layout] : layout
      Wouter.render_template(template, self, engine, dir, layout)
    end
  end
end
